@@ -4,15 +4,16 @@ import asyncio
 # THe Python Interface for JavaScript
 
 import inspect, importlib, traceback
+import threading
 import os, sys, json, types
 import socket
 from typing import Any, Dict, List, Tuple
 
 from .util import generate_snowflake, SnowflakeMode
 from . import proxy, events, config
-from .errors import JavaScriptError, getErrorMessage, NoAsyncLoop
+from .errors import JavaScriptError, getErrorMessage, NoAsyncLoop, NoPyiAction
 from weakref import WeakValueDictionary
-from .logging import logs, log_print
+from .core.jslogging import log_print, log_debug
 
 
 def python(method: str) -> types.ModuleType:
@@ -95,6 +96,10 @@ class Iterate:
 fix_key = lambda key: key.replace("~~", "") if type(key) is str else key
 
 
+
+class PyInterfaceActions:
+    """Mixin which defines actions for PyInterface"""
+
 class PyInterface:
     """
     Python Interface for JavaScript.
@@ -114,13 +119,11 @@ class PyInterface:
 
     """
 
-    def __init__(self, config_obj: config.JSConfig, ipc, exe=None):
+    def __init__(self, config_obj: config.JSConfig):
         """Initalize a new PYInterface.
 
         Args:
             config_obj (config.JSConfig): Reference to the active JSConfig object.
-            ipc(EventLoop): Reference to the event loop.
-            exe: Unused.
 
         """
         self.m = {0: {"python": python, "fileImport": fileImport, "Iterate": Iterate}}
@@ -129,16 +132,31 @@ class PyInterface:
         self.cur_ffid = 10000
         self.ffid_param = 10000
         self.config = config_obj
-        self.ipc: events.EventLoop = ipc
+        self.ipc: events.EventLoop = self.config.get_event_loop()
         # This toggles if we want to send inspect data for console logging. It's auto
         # disabled when a for loop is active; use `repr` to request logging instead.
         self.m[0]["sendInspect"] = lambda x: setattr(self, "send_inspect", x)
         self.send_inspect = True
         self.current_async_loop = None
+        self.my_actions={}
+        self.executor: proxy.Executor = None
+        self.define_actions()
 
-        # self.executor:proxy.Executor = exe
+    def define_actions(self):
+        for name, method in inspect.getmembers(self, predicate=inspect.ismethod):
+            if name!='action_selector':
+                signature = inspect.signature(method)
+                parameters = [k for k in signature.parameters.keys()]
+                #log_debug(name,signature,parameters)
+                if set(["r", "ffid", "key", "args"]).issubset(parameters):
+                    #log_debug("is_subset")
+                    self.my_actions[name] = method
+                if set(["r", "ffid", "keys", "args"]).issubset(parameters):
+                    #log_debug("is_subset2")
+                    self.my_actions[name] = method
+        
 
-    def q(self, r, key, val, sig=""):
+    def queue_push(self, r, key, val, sig=""):
         self.ipc.queue_payload({"c": "pyi", "r": r, "key": key, "val": val, "sig": sig})
 
     def __str__(self):
@@ -146,14 +164,43 @@ class PyInterface:
         res = str(self.m)
         return res
 
-    @property
-    def executor(self):
-        """Get the executor object currently initalized in JSConfig."""
-        return self.config.executor
+    def set_executor(self, exe: proxy.Executor):
+        """Set the current executor object.
 
-    @executor.setter
-    def executor(self, executor):
-        pass
+        Args:
+            exe (proxy.Executor): The new executor object to be set.
+        """
+        self.executor = exe
+
+    # @property
+    # def executor(self):
+    #     """Get the executor object currently initalized in JSConfig."""
+    #     return self.config.executor
+
+    # @executor.setter
+    # def executor(self, executor):
+    #     pass
+
+    def action_selector(self, action: str, r: int, ffid: int, key: str, args: List):
+        """Selects and calls a method based on the given action name.
+
+        Args:
+            action (str): The name of the action corresponding to the method to be called.
+            r (int): An integer argument for the selected method.
+            ffid (int): An integer argument for the selected method.
+            key (str): A string argument for the selected method.
+            args (List): A list of arguments for the selected method.
+
+        Returns:
+            The result of calling the selected method with the given arguments.
+
+        Raises:
+            NoPyiAction: If the given action is not defined in the my_actions dictionary.
+        """
+        if action in self.my_actions:
+            return self.my_actions[action](r, ffid, key, args)
+        raise NoPyiAction(f"There's no method in PYI for action {action}. {r},{ffid},{key},{args}.")
+        
 
     def assign_ffid(self, what: Any):
         """Assign a new FFID (foreign object reference id) for an object.
@@ -199,7 +246,7 @@ class PyInterface:
             else:
                 raise LookupError(f"Property '{fix_key(key)}' does not exist on {repr(v)}")
         l = len(v)
-        self.q(r, "num", l)
+        self.queue_push(r, "num", l)
 
     def init(self, r: int, ffid: int, key: str, args: Tuple):
         """Initialize an object on the Python side, assign an FFID to it, and
@@ -214,7 +261,7 @@ class PyInterface:
         """
         v = self.m[ffid](*args)
         ffid = self.assign_ffid(v)
-        self.q(r, "inst", ffid)
+        self.queue_push(r, "inst", ffid)
 
     def call(self, r: int, ffid: int, keys: List, args: Tuple, kwargs: Dict, invoke=True):
         """Call a method or access a property of an object on the python side,
@@ -238,11 +285,11 @@ class PyInterface:
         # precedence in a dict. However if we're only getting objects, we can
         # first try bracket for dicts, then attributes.
         if invoke:
-            logs.debug("INVOKING MODE %s,%s,%s,%s", v, type(v), str(repr(keys)), str(repr(args)))
+            log_debug("INVOKING MODE %s,%s,%s,%s", v, type(v), str(repr(keys)), str(repr(args)))
             for key in keys:
                 t = getattr(v, str(key), None)
 
-                logs.debug("GET MODE %s,%s,%s,%s", v, type(v), str(key), str(args))
+                log_debug("GET MODE %s,%s,%s,%s", v, type(v), str(key), str(args))
                 if t:
                     v = t
                 elif hasattr(v, "__getitem__"):
@@ -280,32 +327,32 @@ class PyInterface:
             else:
                 if inspect.isclass(v):
                     was_class = True
-                logs.debug("INVOKING %s,%s,%s", v, type(v), was_class)
+                log_debug("INVOKING %s,%s,%s", v, type(v), was_class)
                 v = v(*args, **kwargs)
         typ = type(v)
         if typ is str:
-            self.q(r, "string", v)
+            self.queue_push(r, "string", v)
             return
         if typ is int or typ is float or (v is None) or (v is True) or (v is False):
-            self.q(r, "int", v)
+            self.queue_push(r, "int", v)
             return
         if inspect.isclass(v) or isinstance(v, type):
             # generate a new ffid.
-            self.q(r, "class", self.assign_ffid(v), self.make_signature(v))
+            self.queue_push(r, "class", self.assign_ffid(v), self.make_signature(v))
             return
         if callable(v):  # anything with __call__
-            self.q(r, "fn", self.assign_ffid(v), self.make_signature(v))
+            self.queue_push(r, "fn", self.assign_ffid(v), self.make_signature(v))
             return
         if (typ is dict) or (inspect.ismodule(v)) or was_class:  # "object" in JS speak
-            self.q(r, "obj", self.assign_ffid(v), self.make_signature(v))
+            self.queue_push(r, "obj", self.assign_ffid(v), self.make_signature(v))
             return
         if typ is list:
-            self.q(r, "list", self.assign_ffid(v), self.make_signature(v))
+            self.queue_push(r, "list", self.assign_ffid(v), self.make_signature(v))
             return
         if hasattr(v, "__class__"):  # numpy generator can't be picked up without this
-            self.q(r, "class", self.assign_ffid(v), self.make_signature(v))
+            self.queue_push(r, "class", self.assign_ffid(v), self.make_signature(v))
             return
-        self.q(r, "void", self.cur_ffid)
+        self.queue_push(r, "void", self.cur_ffid)
 
     # Same as call just without invoking anything, and args
     # would be null
@@ -323,8 +370,8 @@ class PyInterface:
             Any: The value of the property.
 
         """
-        o = self.call(r, ffid, keys, [], {}, invoke=False)
-        return o
+        self.call(r, ffid, keys, [], {}, invoke=False)
+        return None
 
     def inspect(self, r: int, ffid: int, keys: List, args: Tuple):
         """Inspect an object and send the representation to NodeJS.
@@ -340,7 +387,7 @@ class PyInterface:
         for key in keys:
             v = getattr(v, key, None) or v[key]
         s = repr(v)
-        self.q(r, "", s)
+        self.queue_push(r, "", s)
 
     # no ACK needed
     def free(self, r: int, ffid: int, key: str, args: List):
@@ -354,14 +401,14 @@ class PyInterface:
             args (List[int]): List of foreign object reference IDs to free.
 
         """
-        logs.debug("free: %s, %s, %s, %s", r, ffid, key, args)
-        logs.debug(str(self))
+        log_debug("free: %s, %s, %s, %s", r, ffid, key, args)
+        log_debug(str(self))
         for i in args:
             if i not in self.m:
                 continue
-            logs.debug(f"purged {i}")
+            log_debug(f"purged {i}")
             del self.m[i]
-        logs.debug(str(self))
+        log_debug(str(self))
 
     def make_signature(self, what: Any) -> str:
         """Generate a signature for an object.
@@ -413,7 +460,23 @@ class PyInterface:
             v[on] = val
         else:
             setattr(v, on, val)
-        self.q(r, "void", self.cur_ffid)
+        self.queue_push(r, "void", self.cur_ffid)
+
+    def json_to_python(self, json_input, lookup_key):
+        """Convert special JSON objects to Python methods"""
+        iterator = None
+        if isinstance(json_input, dict):
+            iterator = json_input.items()
+        elif isinstance(json_input, list):
+            iterator = enumerate(json_input)
+        else:
+            return
+        for k, v in iterator:
+            if isinstance(v, dict) and (lookup_key in v):
+                ffid = v[lookup_key]
+                json_input[k] = proxy.Proxy(self.executor, ffid)
+            else:
+                self.json_to_python(v, lookup_key)
 
     def pcall(self, r: int, ffid: int, key: str, args: Tuple, set_attr: bool = False):
         """Call a method or set a value of an object.
@@ -427,24 +490,7 @@ class PyInterface:
 
         """
 
-        # Convert special JSON objects to Python methods
-        def process(json_input, lookup_key):
-            if isinstance(json_input, dict):
-                for k, v in json_input.items():
-                    if isinstance(v, dict) and (lookup_key in v):
-                        ffid = v[lookup_key]
-                        json_input[k] = proxy.Proxy(self.executor, ffid)
-                    else:
-                        process(v, lookup_key)
-            elif isinstance(json_input, list):
-                for k, v in enumerate(json_input):
-                    if isinstance(v, dict) and (lookup_key in v):
-                        ffid = v[lookup_key]
-                        json_input[k] = proxy.Proxy(self.executor, ffid)
-                    else:
-                        process(v, lookup_key)
-
-        process(args, "ffid")
+        self.json_to_python(args, "ffid")
         pargs, kwargs = args
         if set_attr:
             self.Set(r, ffid, key, pargs)
@@ -489,11 +535,7 @@ class PyInterface:
             else:
                 v = t
 
-        # TODO: do we realy want to worry about functions/classes here?
-        # we're only supposed to send primitives, probably best to ignore
-        # everything else.
-        # payload = json.dumps(v, default=lambda arg: None)
-        self.q(r, "ser", v)
+        self.queue_push(r, "ser", v)
 
     def onMessage(self, r: int, action: str, ffid: int, key: str, args: List):
         """Determine which action to preform based on the
@@ -510,9 +552,10 @@ class PyInterface:
         # current valid acts:
         # length, get, setval,pcall, inspect, value, free
         try:
-            return getattr(self, action)(r, ffid, key, args)
+            return self.action_selector(action,r,ffid,key,args)
+            #return getattr(self, action)(r, ffid, key, args)
         except Exception:
-            self.q(r, "error", "", traceback.format_exc())
+            self.queue_push(r, "error", "", traceback.format_exc())
             pass
 
     def inbound(self, j: Dict[str, Any]):
@@ -522,6 +565,20 @@ class PyInterface:
             j (Dict[str, Any]): The incoming data as a dictionary.
 
         """
-        logs.debug("PYI, %s", j)
+        log_debug("PYI, %s", j)
+        
         # print(j)
+        thread_id = threading.current_thread().ident
+        # print('threadid',thread_id,"INBOUND PYI: ",j)
         return self.onMessage(j["r"], j["action"], j["ffid"], j["key"], j["val"])
+    
+    async def inbound_a(self, j:Dict[str,Any]):
+        """Extract the message arguments from J, and call onMessage.  asyncronous.
+
+        Args:
+            j (Dict[str, Any]): The incoming data as a dictionary.
+
+        """
+        await asyncio.to_thread(self.inbound,j)
+    
+
